@@ -1,6 +1,6 @@
 //! \file request.hpp Defines the Fastcgipp::Request class
 /***************************************************************************
-* Copyright (C) 2007 Eddie                                                 *
+* Copyright (C) 2007 Eddie Carle [eddie@erctech.org]                       *
 *                                                                          *
 * This file is part of fastcgi++.                                          *
 *                                                                          *
@@ -31,19 +31,21 @@
 #include <boost/thread.hpp>
 #include <boost/function.hpp>
 
+#include <fastcgi++/transceiver.hpp>
 #include <fastcgi++/protocol.hpp>
 #include <fastcgi++/exceptions.hpp>
-#include <fastcgi++/transceiver.hpp>
 #include <fastcgi++/fcgistream.hpp>
 #include <fastcgi++/http.hpp>
 
 //! Topmost namespace for the fastcgi++ library
 namespace Fastcgipp
 {
+	template<class T> class Manager;
+
 	//! %Request handling class
 	/*!
 	 * Derivations of this class will handle requests. This
-	 * includes building the session data, processing post/get data,
+	 * includes building the environment data, processing post/get data,
 	 * fetching data (files, database), and producing a response.
 	 * Once all client data is organized, response() will be called.
 	 * At minimum, derivations of this class must define response().
@@ -54,17 +56,21 @@ namespace Fastcgipp
 	 * a 8bit character set encoding pass char as the template argument and
 	 * setloc() a locale with the corresponding character set.
 	 *
-	 * @tparam charT Character type for internal processing (wchar_t or char)
+	 * \tparam charT Character type for internal processing (wchar_t or char)
 	 */
 	template<class charT> class Request
 	{
 	public:
 		//! Initializes what it can. set() must be called by Manager before the data is usable.
-		Request(): state(Protocol::PARAMS)  { setloc(std::locale::classic()); out.exceptions(std::ios_base::badbit | std::ios_base::failbit | std::ios_base::eofbit); session.clearPostBuffer(); }
+		/*!
+		 * \param maxPostSize This would be the maximum size you want to allow for
+		 * post data. Any data beyond this size would result in a call to
+		 * bigPostErrorHandler(). A value of 0 represents unlimited.
+		 */
+		Request(const size_t maxPostSize=0): m_maxPostSize(maxPostSize), state(Protocol::PARAMS)  { setloc(std::locale::classic()); out.exceptions(std::ios_base::badbit | std::ios_base::failbit | std::ios_base::eofbit); m_environment.clearPostBuffer(); }
 
-	protected:
-		//! Structure containing all HTTP session data
-		Http::Session<charT> session;
+		//! Accessor for  the data structure containing all HTTP environment data
+		const Http::Environment<charT>& environment() const { return m_environment; }
 
 		// To dump data into the stream without it being code converted and bypassing the stream buffer call Fcgistream::dump(char* data, size_t size)
 		// or Fcgistream::dump(std::basic_istream<char>& stream)
@@ -73,14 +79,63 @@ namespace Fastcgipp
 		/*!
 		 * To dump data directly through the stream without it being code converted and bypassing the stream buffer call Fcgistream::dump()
 		 */
-		Fcgistream<charT, std::char_traits<charT> > out;
+		Fcgistream<charT> out;
 
 		//! Output stream to the HTTP server error log
 		/*!
 		 * To dump data directly through the stream without it being code converted and bypassing the stream buffer call Fcgistream::dump()
 		 */
-		Fcgistream<charT, std::char_traits<charT> > err;
+		Fcgistream<charT> err;
 
+		//! Called when an exception is caught.
+		/*!
+		 * This function is called whenever an exception is caught inside the request. By default it will output some data
+		 * to the error log and send a standard 500 Internal Server Error message to the user. Override for more specialized
+		 * purposes.
+		 *
+		 * @param[in] error Exception caught
+		 */
+		virtual void errorHandler(const std::exception& error);
+
+		//! Called when too much post data is recieved.
+		virtual void bigPostErrorHandler();
+
+		//! See the requests role
+		Protocol::Role role() const { return m_role; }
+
+		//! Accessor for the callback function for dealings outside the fastcgi++ library
+		/*!
+		 * The purpose of the callback object is to provide a thread safe mechanism for functions and
+		 * classes outside the fastcgi++ library to talk to the requests. Should the library
+		 * wish to have another thread process or fetch some data, that thread can call this
+		 * function when it is finished. It is equivalent to this:
+		 *
+		 * void callback(Message msg);
+		 *
+		 *	The sole parameter is a Message that contains both a type value for processing by response()
+		 *	and the raw castable data.
+		 */
+		const boost::function<void(Message)>& callback() const { return m_callback; }
+
+		//! Set the requests locale
+		/*!
+		 * This function both sets loc to the locale passed to it and imbues the locale into the
+		 * out and err stream. The user should always call this function as opposed to setting the
+		 * locales directly is this functions insures the utf8 code conversion is functioning properly.
+		 *
+		 * @param[in] loc_ New locale
+		 * @sa loc
+		 * @sa out
+		 */
+		void setloc(std::locale loc_){ out.imbue(loc_); err.imbue(loc_); }
+
+		//! Retrieves the requests locale
+		/*!
+		 * \return Constant reference to the requests locale
+		 */
+		const std::locale& getloc(){ return loc; }
+
+	protected:
 		//! Response generator
 		/*!
 		 * This function is called by handler() once all request data has been received from the other side or if a
@@ -98,13 +153,25 @@ namespace Fastcgipp
 		 * bytes received from the client. The function is called by handler() every time a FastCGI IN record is received.
 		 * The function has no access to the data, but knows exactly how much was received based on the value that was passed.
 		 * Note this value represents the amount of data received in the individual record, not the total amount received in
-		 * the session. If the library user wishes to have such a value they would have to keep a tally of all size values
+		 * the environment. If the library user wishes to have such a value they would have to keep a tally of all size values
 		 * passed.
 		 *
 		 * @param[in] bytesReceived Amount of bytes received in this FastCGI record
 		 */
 		virtual void inHandler(int bytesReceived) { };
 
+		//! The message associated with the current handler() call.
+		/*!
+		 * This is only of use to the library user when a non FastCGI (type=0) Message is passed
+		 * by using the requests callback.
+		 *
+		 * @sa callback
+		 */
+		const Message& message() const { return m_message; }
+
+	private:
+		template<class T> friend class Manager;
+
 		//! The locale associated with the request. Should be set with setloc(), not directly.
 		std::locale loc;
 
@@ -115,21 +182,9 @@ namespace Fastcgipp
 		 *
 		 * @sa callback
 		 */
-		Message message;
+		Message m_message;
 
-		//! Set the requests locale
-		/*!
-		 * This function both sets loc to the locale passed to it and imbues the locale into the
-		 * out and err stream. The user should always call this function as opposed to setting the
-		 * locales directly is this functions insures the utf8 code conversion is functioning properly.
-		 *
-		 * @param[in] loc_ New locale
-		 * @sa loc
-		 * @sa out
-		 */
-		void setloc(std::locale loc_);
-
-		//! Callback function for dealings outside the fastcgi++ library
+		//! The callback function for dealings outside the fastcgi++ library
 		/*!
 		 * The purpose of the callback object is to provide a thread safe mechanism for functions and
 		 * classes outside the fastcgi++ library to talk to the requests. Should the library
@@ -141,8 +196,11 @@ namespace Fastcgipp
 		 *	The sole parameter is a Message that contains both a type value for processing by response()
 		 *	and the raw castable data.
 		 */
-		boost::function<void(Message)> callback;
-	private:
+		boost::function<void(Message)> m_callback;
+
+		//! The data structure containing all HTTP environment data
+		Http::Environment<charT> m_environment;
+
 		//! Queue type for pending messages
 		/*!
 		 * This is merely a derivation of a std::queue<Message> and a
@@ -152,6 +210,9 @@ namespace Fastcgipp
 		//! A queue of messages to be handler by the request
 		Messages messages;
 
+		//! The maximum amount of post data that can be recieved
+		const size_t m_maxPostSize;
+
 		//! Request Handler
 		/*!
 		 * This function is called by Manager::handler() to handle messages destined for the request.
@@ -161,11 +222,10 @@ namespace Fastcgipp
 		 * @sa callback
 		 */
 		bool handler();
-		template <typename T> friend class Manager;
 		//! Pointer to the transceiver object that will send data to the other side
 		Transceiver* transceiver;
 		//! The role that the other side expects this request to play
-		Protocol::Role role;
+		Protocol::Role m_role;
 		//! The complete ID (request id & file descriptor) associated with the request
 		Protocol::FullId id;
 		//! Boolean value indicating whether or not the file descriptor should be closed upon completion.
@@ -189,13 +249,33 @@ namespace Fastcgipp
 			killCon=killCon_;
 			id=id_;
 			transceiver=&transceiver_;
-			role=role_;
-			callback=callback_;
+			m_role=role_;
+			m_callback=callback_;
 
 			err.set(id_, transceiver_, Protocol::ERR);
 			out.set(id_, transceiver_, Protocol::OUT);
 		}
 	};
+
+	//! Includes all exceptions used by the fastcgi++ library
+	namespace Exceptions
+	{
+		/** 
+		 * @brief Thrown if FastCGI records are received out of order.
+		 */
+		struct RecordsOutOfOrder: public std::exception
+		{
+			const char* what() const throw() { return "FastCGI records received out of order from server."; }
+		};
+
+		/** 
+		 * @brief Thrown if a incoming content type is unknown
+		 */
+		struct UnknownContentType: public std::exception
+		{
+			const char* what() const throw() { return "Client sent unknown content type."; }
+		};
+	}
 }
 
 #endif
